﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Post11443
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private ExTrackInteractiveOverlay trackOverlay;
        private bool drawMovePoint = false;
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-135.7, 83.6, 113.5, -53);
            trackOverlay = new ExTrackInteractiveOverlay(GeographyUnit.DecimalDegree);
            wpfMap1.TrackOverlay = trackOverlay;
            wpfMap1.TrackOverlay.MapMouseMove += new EventHandler<MapMouseMoveInteractiveOverlayEventArgs>(TrackOverlay_MapMouseMove);
            wpfMap1.TrackOverlay.VertexAdding += new EventHandler<VertexAddingTrackInteractiveOverlayEventArgs>(TrackOverlay_VertexAdding);
            wpfMap1.TrackOverlay.MapMouseDoubleClick += new EventHandler<MapMouseDoubleClickInteractiveOverlayEventArgs>(TrackOverlay_MapMouseDoubleClick);
        }

        void TrackOverlay_MapMouseMove(object sender, MapMouseMoveInteractiveOverlayEventArgs e)
        {
            if (drawMovePoint)
            {
                if (!wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Contains("movePoint"))
                {
                    Feature movePointFeature = new Feature(new PointShape(e.InteractionArguments.WorldX, e.InteractionArguments.WorldY));
                    movePointFeature.ColumnValues.Add(trackOverlay.ColumnKey, "X:" + e.InteractionArguments.WorldX.ToString() + " Y:" + e.InteractionArguments.WorldY.ToString());
                    wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add("movePoint", movePointFeature);
                }
                else
                {
                    wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Remove("movePoint");
                    Feature movePointFeature = new Feature(new PointShape(e.InteractionArguments.WorldX, e.InteractionArguments.WorldY));
                    movePointFeature.ColumnValues.Add(trackOverlay.ColumnKey, "X:" + e.InteractionArguments.WorldX.ToString() + " Y:" + e.InteractionArguments.WorldY.ToString());
                    wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add("movePoint", movePointFeature);
                }
            }
        }

        void TrackOverlay_MapMouseDoubleClick(object sender, MapMouseDoubleClickInteractiveOverlayEventArgs e)
        {
            InMemoryFeatureLayer trackShapeLayer = wpfMap1.TrackOverlay.TrackShapeLayer;
            Feature point = new Feature(e.InteractionArguments.WorldX, e.InteractionArguments.WorldY);
            point.ColumnValues.Add(trackOverlay.ColumnKey, "X:" + e.InteractionArguments.WorldX + " Y:" + e.InteractionArguments.WorldY);
            trackShapeLayer.InternalFeatures.Add(point);
        }


        void TrackOverlay_VertexAdding(object sender, VertexAddingTrackInteractiveOverlayEventArgs e)
        {
            InMemoryFeatureLayer trackShapeLayer = wpfMap1.TrackOverlay.TrackShapeLayer;
            Feature point = new Feature(e.AddingVertex.X, e.AddingVertex.Y);
            point.ColumnValues.Add(trackOverlay.ColumnKey, "X:" + e.AddingVertex.X + " Y:" + e.AddingVertex.Y);
            trackShapeLayer.InternalFeatures.Add(point);
        }

        private void btnTrack_Click(object sender, RoutedEventArgs e)
        {
            Button btn = sender as Button;
            if (btn != null)
            {
                switch (btn.Name)
                {
                    case "btnTrackPolygon":
                        drawMovePoint = true;
                        wpfMap1.TrackOverlay.TrackMode = TrackMode.Polygon;
                        break;
                    case "btnTrackNone":
                        drawMovePoint = false;
                        //add the EditOverlay fetures to TrackOverlay, and then clear the fetures in EditOverlay
                        foreach (Feature feature in wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures)
                        {
                            wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(feature);
                        }
                        wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                        wpfMap1.Refresh(new Overlay[] { wpfMap1.EditOverlay, wpfMap1.TrackOverlay });
                        if (wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Count > 0)
                        {
                            wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Remove("movePoint");
                        }
                        wpfMap1.TrackOverlay.TrackMode = TrackMode.None;
                        break;
                    case "btnEdit"://add a edit state
                        drawMovePoint = false;
                        wpfMap1.TrackOverlay.TrackMode = TrackMode.None;
                        //add the TrackOverlay fetures to EditOverlay, and then clear the fetures in TrackOverlay
                        foreach (Feature feature in wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures)
                        {
                            wpfMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                        }
                        wpfMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
                        wpfMap1.EditOverlay.CalculateAllControlPoints();//make it to edit
                        wpfMap1.Refresh(new Overlay[] { wpfMap1.EditOverlay, wpfMap1.TrackOverlay });
                        break;
                }
            }
        }
    }

    public class ExTrackInteractiveOverlay : TrackInteractiveOverlay
    {
        private readonly GeographyUnit _mapUnit;
        public readonly string ColumnKey = "length";

        public ExTrackInteractiveOverlay(GeographyUnit mapUnit)
            : base()
        {
            this._mapUnit = mapUnit;

            TrackShapeLayer.Open();
            TrackShapeLayer.Columns.Add(new FeatureSourceColumn(ColumnKey));
            TrackShapeLayer.Close();
            TrackShapeLayer.ZoomLevelSet = GetZoomLevelSet();
        }

        protected override void DrawCore(RectangleShape targetExtent, OverlayRefreshType overlayRefreshType)
        {
            if (TrackShapeLayer.InternalFeatures.Any())
            {
                if (!TrackShapeLayer.InternalFeatures.Any()) return;
                Feature feature = TrackShapeLayer.InternalFeatures.Last(f => f.GetWellKnownType().Equals(WellKnownType.Polygon));
                BaseShape shape = feature.GetShape();
                feature.ColumnValues.Clear();
                var box = shape.GetBoundingBox();
                switch (shape.GetWellKnownType())
                {
                    case WellKnownType.Polygon:
                        {
                            double area = ((PolygonShape)shape).GetArea(_mapUnit, AreaUnit.SquareMeters);
                            feature.ColumnValues.Add(ColumnKey, string.Format("{0:n0}", (Int64)area) + " Square Meter");
                        }
                        break;
                    default:
                        return;
                }
            }
            base.DrawCore(targetExtent, overlayRefreshType);
        }

        public ZoomLevelSet GetZoomLevelSet()
        {
            var style = new ZoomLevelSet();
            style.ZoomLevel01.DefaultLineStyle = LineStyles.Highway1;
            style.ZoomLevel01.DefaultAreaStyle = AreaStyles.State1;
            style.ZoomLevel01.DefaultTextStyle = TextStyles.City1(ColumnKey);
            //TODO:Hi,You can use image as a point.
            style.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            style.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(@"..\App_Data\Point.png");
            style.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            return style;
        }
    }
}
